/**
   @file plugin.c

   Osso Lmarbles plugin functions.

    Copyright (c) 2004, 2005 Nokia Corporation.
	
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA
*/

#include <stdio.h>
#include <libintl.h>
#include <gtk/gtk.h>
#include <gconf/gconf.h>
#include <gconf/gconf-client.h>
#include <hildon-widgets/hildon-note.h>
#include <startup_plugin.h>
#include <osso-helplib.h>
#include <hildon-widgets/hildon-caption.h>
#include <gdk/gdkkeysyms.h>


#define _(String) dgettext("osso-games", String)
#define __(String) dgettext("osso-games", String)

#define OSSO_MARBLES_HELP_PATH "//marbles/a/1"

#define LMARBLES_ENABLE_SOUND           "/apps/osso/lmarbles/enable_sound"
#define LMARBLES_DIFFICULTY_LEVEL       "/apps/osso/lmarbles/difficulty_level"

#define LMARBLES_SETTINGS_EASY          0
#define LMARBLES_SETTINGS_NORMAL        1
#define LMARBLES_SETTINGS_HARD          2
#define LMARBLES_SETTINGS_BRAINSTORM    3
#define LMARBLES_SETTINGS_SOUND         4
#define LMARBLES_SETTINGS_RESET         5
#define MA_GAME_PLAYING_START 30
#define MA_GAME_PLAYING 31


#define MA_GAME_RESET_OLD_SETTINGS 35

GConfClient *gcc = NULL;
GtkWidget *sound_check = NULL;
GtkWidget *difficult_box = NULL;

static void       lmarbles_sound_cb        (GtkWidget *widget, gpointer data);
static void       lmarbles_difficulty_cb   (GtkWidget *widget, gpointer data);

static GtkWidget *load_plugin      (void);
static void       unload_plugin    (void);
static void       write_config     (void);
static GtkWidget **load_menu       (guint *);
static void       update_menu      (void);
static void 			plugin_game_help_cb (GtkMenuItem *menuitem, gpointer user_data);

static void       plugin_callback  (GtkWidget *menu_item, gpointer   data);
static gint 			plugin_settings_get_int_fallback(gchar *key, gint fall);
static GConfValue *plugin_settings_get(const gchar *key);
static void       plugin_cb                         (GtkWidget   *menu_item,
                                                     gpointer     cb_data);
static gboolean   plugin_ui_show_reset_game_progress_dialog(void);
static gboolean   plugin_dialog_key_press(GtkWidget *widget, GdkEventKey *event, gpointer data);
static gboolean   plugin_settings_get_bool_fallback(gchar *key, gboolean fall);
static gboolean   plugin_settings_get_bool(const gchar *key);
static void set_all_insensitive(void);
static void set_all_sensitive(void);
static void plugin_show_infoprint(gchar *msg);

static StartupPluginInfo plugin_info = {
  load_plugin,
  unload_plugin,
  write_config,
  load_menu,
  update_menu,
  plugin_cb
};

typedef struct _StartupApp        StartupApp;
struct _StartupApp {
  StartupConfig *config;
  osso_context_t *osso;
  guint state;

  startup_app_top_cb *top_cb;
  gpointer top_cb_data;
  startup_app_state_change_cb *state_change_cb;
  gpointer state_change_cb_data;

  const gchar *service;
  const gchar *path;
  const gchar *iface;
};

static int changed = FALSE;
static GameStartupInfo gs;
static GtkWidget *menu_items[2];

STARTUP_INIT_PLUGIN(plugin_info, gs, FALSE, FALSE)

static void plugin_show_infoprint(gchar *msg)
{
  
    gtk_infoprint( GTK_WINDOW(gs.ui->hildon_app), msg);
}


static gboolean plugin_dialog_key_press(GtkWidget *widget, GdkEventKey *event, gpointer data)
{
  (void)data;
  
  if (event->state & (GDK_CONTROL_MASK |
                      GDK_SHIFT_MASK |
                      GDK_MOD1_MASK |
                      GDK_MOD3_MASK |
                      GDK_MOD4_MASK |
                      GDK_MOD5_MASK)) {
    return FALSE;
  }

  switch (event->keyval)
  {
    case GDK_Escape:
        gtk_dialog_response(GTK_DIALOG(widget), FALSE);
      return TRUE;
      break;
    default:
      break;
  }
  
  return FALSE;
}


/* Get int type key value */
static gint plugin_settings_get_int(const gchar *key)
{
  return gconf_client_get_int(gcc, key, NULL);
}


/* Get key value */
static GConfValue *plugin_settings_get(const gchar *key)
{
  return gconf_client_get(gcc, key, NULL);
}

/* Get boolean type key value */
static gboolean plugin_settings_get_bool(const gchar *key)
{
  return gconf_client_get_bool(gcc, key, NULL);
}


/**
 Helper function to get an bool entry, returning defined value, if not found.
 @param key GConf key to be get.
 @param fall Fallback to this, if not found, or invalid.
 @return Value got from GConf or the value specified in fall.
*/
static gboolean plugin_settings_get_bool_fallback(gchar *key, gboolean fall)
{
  GConfValue *gc_val = plugin_settings_get(key);
  
  if (gc_val)
  {
    if (gc_val->type == GCONF_VALUE_BOOL) {
      g_free(gc_val);
      return plugin_settings_get_bool(key);
    }
    g_free(gc_val);
  }
  return fall;
}


static gint plugin_settings_get_int_fallback(gchar *key, gint fall)
{
  GConfValue *gc_val = plugin_settings_get(key);
  if (gc_val)
  {
    if (gc_val->type == GCONF_VALUE_INT) {
      g_free(gc_val);
      return plugin_settings_get_int(key);
    }
    g_free(gc_val);
  }
  return fall;
}

static GtkWidget *
load_plugin (void)
{
  int sound, difficulty;
  GtkWidget *game_hbox;
  GtkWidget *difficult_label;
  GtkWidget *sound_label;

  gcc = gconf_client_get_default ();
  sound = plugin_settings_get_bool_fallback( LMARBLES_ENABLE_SOUND, TRUE);
  difficulty = plugin_settings_get_int_fallback(LMARBLES_DIFFICULTY_LEVEL, 1);

  game_hbox = gtk_hbox_new (FALSE, 0);
  g_assert (game_hbox);

  //difficult_hbox = gtk_hbox_new (FALSE, 4);
  
  difficult_box = gtk_combo_box_new_text ();
  g_assert (difficult_box);

  gtk_combo_box_append_text (GTK_COMBO_BOX (difficult_box),
    __("game_va_lmarbles_settings_difficulty_1"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (difficult_box),
    __("game_va_lmarbles_settings_difficulty_2"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (difficult_box),
    __("game_va_lmarbles_settings_difficulty_3"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (difficult_box),
    __("game_va_lmarbles_settings_difficulty_4"));
  gtk_combo_box_set_active (GTK_COMBO_BOX (difficult_box), difficulty);

  gtk_widget_set_size_request (difficult_box,
    200, -1);
  /*gtk_box_pack_end (GTK_BOX (difficult_hbox),
    difficult_box, FALSE, FALSE, 0);*/
/*		GtkWidget *mahjong_dummy_vbox;
  mahjong_dummy_vbox = gtk_hbox_new( FALSE, 0 );
  if (mahjong_dummy_vbox != NULL)
    gtk_box_pack_end( GTK_CONTAINER (game_hbox ),
      mahjong_dummy_vbox, FALSE, FALSE , 2 );*/

  //difficult_label = gtk_label_new (__("game_fi_lmarbles_settings_difficulty"));
	difficult_label = hildon_caption_new(NULL,__("game_fi_lmarbles_settings_difficulty"),difficult_box,NULL,HILDON_CAPTION_OPTIONAL);
	g_assert(difficult_label);
  //hildon_caption_set_separator(HILDON_CAPTION(difficult_label)," ");
  

  /*gtk_box_pack_end (GTK_BOX (difficult_hbox),
    difficult_label, FALSE, FALSE, 0);*/

  gtk_box_pack_start (GTK_BOX (game_hbox),
    difficult_label, FALSE, FALSE, 2);

  //sound_hbox = gtk_hbox_new (FALSE, 4);
  
  //sound_label = gtk_label_new (__("game_fi_lmarbles_settings_sound"));
	
  

  
  sound_check = gtk_check_button_new ();
  g_assert (sound_check);
	sound_label = hildon_caption_new(NULL,__("game_fi_lmarbles_settings_sound"),sound_check,NULL,HILDON_CAPTION_OPTIONAL);
	g_assert (sound_label);
  //hildon_caption_set_separator(HILDON_CAPTION(sound_label)," ");
	
	gtk_box_pack_end (GTK_BOX (game_hbox),
    sound_label, FALSE, FALSE, 2);


  /*gtk_box_pack_start (GTK_BOX (sound_hbox),
    sound_check, FALSE, FALSE, 0);*/

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(sound_check), sound);

  /*gtk_box_pack_start (GTK_BOX (game_hbox),
    sound_hbox, FALSE, FALSE, 2);*/
 
  g_signal_connect(G_OBJECT(difficult_box), "changed",
   G_CALLBACK(lmarbles_difficulty_cb), NULL);	

  g_signal_connect(G_OBJECT(sound_check), "clicked",
   G_CALLBACK(lmarbles_sound_cb), NULL);

  return game_hbox;
}

static gboolean plugin_ui_show_reset_game_progress_dialog(void)
{
  gboolean answer = FALSE;
  HildonNote *note = HILDON_NOTE(hildon_note_new_confirmation_add_buttons(
        GTK_WINDOW (gs.ui->hildon_app),
        __("game_nc_reset_game_progress"),
        __("game_bd_reset_game_progress_ok"),
        TRUE,
        __("game_bd_reset_game_progress_cancel"),
        FALSE,
        NULL));
  gtk_widget_show_all(GTK_WIDGET(note)) ;
  g_signal_connect(G_OBJECT(note), "key_press_event",
    G_CALLBACK(plugin_dialog_key_press), NULL);
  answer = gtk_dialog_run(GTK_DIALOG(note));
  gtk_widget_destroy(GTK_WIDGET(note));
  return answer;
}

static void
unload_plugin (void)
{
}

static void 
write_config (void)
{
  gconf_client_set_bool(gcc, LMARBLES_ENABLE_SOUND, 
  	gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (sound_check)), NULL);
  gconf_client_set_int(gcc, LMARBLES_DIFFICULTY_LEVEL,
  	gtk_combo_box_get_active (GTK_COMBO_BOX (difficult_box)), NULL);
}

static void plugin_game_help_cb (GtkMenuItem *menuitem, gpointer user_data)
{
	StartupApp *app=NULL;
	app=gs.ui->app;
  ossohelp_show(app->osso,
      OSSO_MARBLES_HELP_PATH,
      0);
}

GtkWidget *difficulty_rd[4], *sound_ck;

static GtkWidget **
load_menu (guint *nitems) {
  GSList *group = NULL;
  GtkWidget *settings_menu=NULL, *reset_progress=NULL,*reset_menu=NULL,*reset_submenu=NULL;

  *nitems = 2;

  menu_items[0] = gtk_menu_item_new_with_label (__("game_me_lmarbles_main_menu_settings"));
  settings_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_items[0]), settings_menu);

  difficulty_rd[0] = gtk_radio_menu_item_new_with_label (group, __("game_me_lmarbles_menu_settings_easy"));
  group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(difficulty_rd[0]));
  gtk_menu_append (GTK_MENU (settings_menu), difficulty_rd[0]);
  g_signal_connect (G_OBJECT (difficulty_rd[0]), "toggled", G_CALLBACK (plugin_callback), (gpointer)LMARBLES_SETTINGS_EASY);

  difficulty_rd[1] = gtk_radio_menu_item_new_with_label (group, __("game_me_lmarbles_menu_settings_normal"));
  group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(difficulty_rd[1]));
  gtk_menu_append (GTK_MENU (settings_menu), difficulty_rd[1]);
  g_signal_connect (G_OBJECT (difficulty_rd[1]), "toggled", G_CALLBACK (plugin_callback), (gpointer)LMARBLES_SETTINGS_NORMAL);

  difficulty_rd[2] = gtk_radio_menu_item_new_with_label (group, __("game_me_lmarbles_menu_settings_hard"));
  group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(difficulty_rd[2]));
  gtk_menu_append (GTK_MENU (settings_menu), difficulty_rd[2]);
  g_signal_connect (G_OBJECT (difficulty_rd[2]), "toggled", G_CALLBACK (plugin_callback), (gpointer)LMARBLES_SETTINGS_HARD);

  difficulty_rd[3] = gtk_radio_menu_item_new_with_label (group, __("game_me_lmarbles_menu_settings_brainstorm"));
  gtk_menu_append (GTK_MENU (settings_menu), difficulty_rd[3]);
  g_signal_connect (G_OBJECT (difficulty_rd[3]), "toggled", G_CALLBACK (plugin_callback), (gpointer)LMARBLES_SETTINGS_BRAINSTORM);

  gtk_menu_append (GTK_MENU (settings_menu), gtk_menu_item_new());

  sound_ck = gtk_check_menu_item_new_with_label (__("game_me_lmarbles_menu_settings_sound"));
  gtk_menu_append (GTK_MENU (settings_menu), sound_ck);
  g_signal_connect (G_OBJECT (sound_ck), "toggled", G_CALLBACK (plugin_callback), (gpointer)LMARBLES_SETTINGS_SOUND);
            
  gtk_menu_append (GTK_MENU (settings_menu), gtk_menu_item_new());

  reset_submenu = gtk_menu_item_new_with_label (__("game_me_lmarbles_menu_settings_reset"));
  reset_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (reset_submenu), reset_menu);
  reset_progress = gtk_menu_item_new_with_label (__("game_me_lmarbles_menu_settings_reset_progress"));
  gtk_menu_append (GTK_MENU (settings_menu), reset_submenu);
  gtk_menu_append (GTK_MENU (reset_menu), reset_progress);
	g_signal_connect (G_OBJECT (reset_progress), "activate", G_CALLBACK (plugin_callback), (gpointer)LMARBLES_SETTINGS_RESET);
	
	menu_items[1] = gtk_menu_item_new_with_label (_("game_me_lmarbles_main_menu_help"));
	g_signal_connect (G_OBJECT (menu_items[1]), "activate",
                      G_CALLBACK (plugin_game_help_cb),
                      NULL);
	
	
  

  return menu_items;
}

static void set_all_insensitive(void)
{
  gtk_widget_set_sensitive(sound_check,FALSE);
	gtk_widget_set_sensitive(difficult_box,FALSE);
	if (GTK_WIDGET_VISIBLE(gs.ui->play_button))
	{
		gtk_widget_set_sensitive(gs.ui->play_button,FALSE);
	} 
	if (GTK_WIDGET_VISIBLE(gs.ui->restart_button))
	{
		gtk_widget_set_sensitive(gs.ui->restart_button,FALSE);
	}

}

static void set_all_sensitive(void)
{
  gtk_widget_set_sensitive(sound_check,TRUE);
	gtk_widget_set_sensitive(difficult_box,TRUE);
}

static void plugin_restore_original_settings(void)
{
  gint original_settings=plugin_settings_get_int_fallback(LMARBLES_DIFFICULTY_LEVEL, 1);
  gint current_value=gtk_combo_box_get_active(GTK_COMBO_BOX( difficult_box ));
  if (original_settings!=current_value)
  {
    plugin_show_infoprint(_("game_ib_changes_effect_next_game"));
    gtk_combo_box_set_active (GTK_COMBO_BOX (difficult_box), original_settings);
  }
  
}

static void 
plugin_cb(GtkWidget *menu_item, gpointer cb_data)
{
  switch( (int) cb_data ) {
    case MA_GAME_RESET_OLD_SETTINGS:
		{
			plugin_restore_original_settings();
			break;
		}
    case MA_GAME_PLAYING_START:
    {
      set_all_insensitive();
      break; 
    }
	  case MA_GAME_PLAYING:
    {
      set_all_sensitive();
      break; 
    }
	}
}

void 
plugin_callback (GtkWidget *menu_item, gpointer data)
{
  char filename[128];
  
  switch ((int)data) {
    case LMARBLES_SETTINGS_EASY:
    case LMARBLES_SETTINGS_NORMAL:
    case LMARBLES_SETTINGS_HARD:
    case LMARBLES_SETTINGS_BRAINSTORM:
      if (!changed) {
        changed = TRUE;
        gtk_combo_box_set_active (GTK_COMBO_BOX (difficult_box), (int)data - LMARBLES_SETTINGS_EASY);
        changed = FALSE;
      }
      break;

    case LMARBLES_SETTINGS_SOUND:
      if (!changed) {
        changed = TRUE;
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (sound_check),
          gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(sound_ck)));
        changed = FALSE;
      }
      break;
    case LMARBLES_SETTINGS_RESET:
      if (plugin_ui_show_reset_game_progress_dialog()==TRUE) {
        sprintf(filename, "%s/.lmarbles_profile", (char*)getenv("HOME"));
        remove(filename);
        sprintf(filename, "%s/.lmarbles_state", (char*)getenv("HOME"));
        remove(filename);
        gs.startup_ui_state_change_cb(NULL, 0, 0, gs.ui);
      }
      break;
  }
}

static void 
update_menu (void)
{
	lmarbles_sound_cb (sound_check, NULL);
	lmarbles_difficulty_cb (difficult_box, NULL);
}

/* Set menu item activity */
static void 
lmarbles_difficulty_cb (GtkWidget *widget, 
                        gpointer   data)
{
  int active;

  if (!changed) {
    changed = TRUE;
    active = gtk_combo_box_get_active(GTK_COMBO_BOX( widget ));
    if (active >= 0 && active < 4)
      gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (difficulty_rd[active]), TRUE);
    changed = FALSE;
  } 
}

static void 
lmarbles_sound_cb (GtkWidget *widget, gpointer data)
{
  if (!changed) {
    changed = TRUE;
    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (sound_ck), 
      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)));
    changed = FALSE;
  }
}
